# -*- coding: utf-8 -*-
#
# Copyright (C) 2022-2024 Graz University of Technology.
#
# invenio-moodle is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Command-line interface for `invenio_moodle`."""

from click import STRING, group, option, secho
from click_params import URL
from flask import current_app
from flask.cli import with_appcontext
from invenio_access import any_user
from invenio_access.utils import get_identity
from invenio_accounts import current_accounts

from .services import MoodleRESTService, build_service
from .types import Color


@group()
def moodle() -> None:
    """invenio-moodle commands."""


@moodle.command("import")
@option("--endpoint", type=URL, required=True)
@option("--user-email", type=STRING, required=True)
@option("--dry-run", is_flag=True, default=False)
@build_service
@with_appcontext
def import_by_endpoint(
    moodle_service: MoodleRESTService,
    user_email: str,
    *,
    dry_run: bool,
) -> None:
    """Fetch data from MOODLE_FETCH_URL and insert it into the database."""
    import_func = current_app.config.get("MOODLE_REPOSITORY_IMPORT_FUNC")

    try:
        user = current_accounts.datastore.get_user_by_email(user_email)
        identity = get_identity(user)
        identity.provides.add(any_user)
    except AttributeError:
        secho("The given user has not been found in the database", fg="red")
        return

    try:
        records = moodle_service.fetch_records(identity)
    except RuntimeError as error:
        secho(str(error), fg=Color.error)
        return

    for moodle_record in records:
        try:
            import_func(identity, moodle_record, moodle_service, dry_run=dry_run)
        except RuntimeError as error:
            secho(error, fg=Color.error)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022-2024 Graz University of Technology.
#
# invenio-moodle is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Types."""

from dataclasses import dataclass
from pathlib import Path

URL = str
"""Type to indicate that an URL is necessary."""

FilePath = str
"""The path to the file on the local machine."""

MoodleID = str
"""Moodle id is the content hash of the file."""


@dataclass(frozen=True)
class Color:
    """The class is for the output color management."""

    neutral = "white"
    error = "red"
    warning = "yellow"
    abort = "magenta"
    success = "green"
    alternate = ("blue", "cyan")


@dataclass(frozen=True)
class FileCacheInfo:
    """Holds a file-path and the file's md5-hash."""

    hash_md5: str
    path: Path



# -*- coding: utf-8 -*-
#
# Copyright (C) 2025 Graz University of Technology.
#
# invenio-moodle is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Jobs."""


from invenio_jobs.jobs import JobType

from .tasks import import_records


class ImportMoodleRecordsJob(JobType):
    """Import moodle records."""

    id = "import_moodle_records"
    title = "Import Moodle Records"
    description = "Import moodle records."

    task = import_records



# -*- coding: utf-8 -*-
#
# Copyright (C) 2024 Graz University of Technology.
#
# invenio-moodle is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Services."""

from collections.abc import Callable
from functools import wraps

from flask_principal import Identity
from marshmallow import ValidationError

from .records import MoodleAPI, MoodleRESTConfig
from .schemas import MoodleSchemaApplicationProfile1, MoodleSchemaApplicationProfile2
from .types import FileCacheInfo
from .utils import extract_moodle_records, post_processing


class MoodleRESTServiceConfig(MoodleRESTConfig):
    """Rest config."""

    api_cls = MoodleAPI


class MoodleRESTService:
    """Moodle rest service."""

    def __init__(self, config: MoodleRESTServiceConfig) -> None:
        """Construct."""
        self._config = config
        self.api = self.api_cls(config=config)

    @property
    def api_cls(self) -> MoodleAPI:
        """Get api cls."""
        return self._config.api_cls

    def download_file(self, _: Identity, url: str) -> FileCacheInfo:
        """Download file."""
        return self.api.download_file(url)

    def fetch_records(self, _: Identity) -> list[dict]:
        """Fetch moodle."""
        moodle_data = self.api.fetch_records()

        try:
            MoodleSchemaApplicationProfile1().load(moodle_data)
        except ValidationError:
            try:
                MoodleSchemaApplicationProfile2().load(moodle_data)
            except ValidationError as error:
                raise RuntimeError(str(error)) from error

        moodle_records = extract_moodle_records(moodle_data)
        post_processing(moodle_records)

        return moodle_records


def build_service[T](func: Callable[..., T]) -> Callable:
    """Decorate to build the services."""

    @wraps(func)
    def build(*_: dict, **kwargs: dict) -> T:
        endpoint = kwargs.pop("endpoint")
        config = MoodleRESTServiceConfig(endpoint)
        kwargs["moodle_service"] = MoodleRESTService(config)

        return func(**kwargs)

    return build



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022-2024 Graz University of Technology.
#
# invenio-moodle is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Flask extension for invenio-moodle."""

from flask import Flask

from . import config
from .services import MoodleRESTService, MoodleRESTServiceConfig


class InvenioMoodle:
    """invenio-moodle extension."""

    def __init__(self, app: Flask = None) -> None:
        """Extension initialization."""
        if app:
            self.init_app(app)

    def init_app(self, app: Flask) -> None:
        """Flask application initialization."""
        self.init_config(app)
        self.init_services(app)
        app.extensions["invenio-moodle"] = self

    def init_config(self, app: Flask) -> None:
        """Init config."""
        for k in dir(config):
            app.config.setdefault(k, getattr(config, k))

    def init_services(self, app: Flask) -> None:
        """Init Services."""
        endpoint = app.config.get("MOODLE_ENDPOINT", "")
        config = MoodleRESTServiceConfig(endpoint)
        self.moodle_rest_service = MoodleRESTService(config)
